describe('MessageService', function() {

	var zhPromise = new ZHPromise(Q);
	var messageService;
	var messages = {
		'TEST#CODE1': 'This is the test number: __number__'
	};
	var screenServiceMock;
	var metaDataLoader = {
		loadMessageByCode: function() {
			return zhPromise.when(messages);
		}
	};

	beforeEach(function() {
		screenServiceMock = jasmine.createSpyObj([
			"showMessage",
			"confirmMessage",
			"showCustomDialog",
			"notificationMessage",
			"alertNotification",
			"successNotification",
			"errorNotification"
		]);

		messageService = new MessageService(screenServiceMock, metaDataLoader, zhPromise);
	});

	it("showMessageByCode should reject when it can't find the message", function(done) {
		messageService.getMessageByCode('TEST#CODE0').then(function() {
			fail("The promise should have been rejected");
		}).catch(function() {
			expect(screenServiceMock.showMessage).not.toHaveBeenCalled();
			done();
		});
	});

	it('showMessageByCode should get the correct message', function(done) {
		messageService.showMessageByCode('TEST#CODE1', {
			number: 10
		}).then(function() {
			var message = screenServiceMock.showMessage.calls.mostRecent().args[0];
			var number = screenServiceMock.showMessage.calls.mostRecent().args[3].number;
			expect(message).toEqual('This is the test number: __number__');
			expect(number).toEqual(10);
			done();
		});
	});

	it('confirmMessageByCode should get the correct message', function(done) {
		messageService.confirmMessageByCode('TEST#CODE1', {
			number: 20
		}).then(function() {
			var message = screenServiceMock.confirmMessage.calls.mostRecent().args[0];
			var number = screenServiceMock.confirmMessage.calls.mostRecent().args[5].number;
			expect(message).toEqual('This is the test number: __number__');
			expect(number).toEqual(20);
			done();
		});
	});

	it('showCustomDialogByCode should get the correct message', function(done) {
		var message;

		messageService.showCustomDialogByCode('TEST#CODE1', {
			number: 30
		}).then(function() {
			var message = screenServiceMock.showCustomDialog.calls.mostRecent().args[0];
			var number = screenServiceMock.showCustomDialog.calls.mostRecent().args[4].number;
			expect(message).toEqual('This is the test number: __number__');
			expect(number).toEqual(30);
			done();
		});
	});

	it('notificationMessageByCode should get the correct message', function(done) {
		messageService.notificationMessageByCode('TEST#CODE1', {
			number: 40
		}, {}).then(function() {
			var message = screenServiceMock.notificationMessage.calls.mostRecent().args[0];
			var number = screenServiceMock.notificationMessage.calls.mostRecent().args[3].number;
			expect(message).toEqual('This is the test number: __number__');
			expect(number).toEqual(40);
			done();
		});
	});

	it('alertNotificationByCode should get the correct message', function(done) {
		messageService.alertNotificationByCode('TEST#CODE1', 0, {
			number: 50
		}).then(function() {
			var message = screenServiceMock.alertNotification.calls.mostRecent().args[0];
			var number = screenServiceMock.alertNotification.calls.mostRecent().args[2].number;
			expect(message).toEqual('This is the test number: __number__');
			expect(number).toEqual(50);
			done();
		});
	});

	it('successNotificationByCode should get the correct message', function(done) {
		messageService.successNotificationByCode('TEST#CODE1', 0, {
			number: 60
		}).then(function() {
			var message = screenServiceMock.successNotification.calls.mostRecent().args[0];
			var number = screenServiceMock.successNotification.calls.mostRecent().args[2].number;
			expect(message).toEqual('This is the test number: __number__');
			expect(number).toEqual(60);
			done();
		});
	});

	it('errorNotificationByCode should get the correct message', function(done) {
		messageService.errorNotificationByCode('TEST#CODE1', {
			number: 70
		}).then(function() {
			var message = screenServiceMock.errorNotification.calls.mostRecent().args[0];
			var number = screenServiceMock.errorNotification.calls.mostRecent().args[1].number;
			expect(message).toEqual('This is the test number: __number__');
			expect(number).toEqual(70);
			done();
		});
	});
});