describe('ZhEventDirectives', function() {
  var $compile,
      $rootScope,
      $document,
      $templateManager,
      $screenService,
      $timeout;
  // Load the myApp module, which contains the directive
  beforeEach(function(){
    window.templateUrl = "../../dyman/templates/dyman/";
    window.metadataUrl = "/json_test/test";
    window.serviceUrl = "../backend/service/index.php";    
    module('ZeedhiApp');
  });

  // Store references to $rootScope and $compile
  // so they are available to all tests in this describe block
  beforeEach(inject(function(_$compile_, _$rootScope_, _$controller_, _$document_, templateManager, ScreenService, _$timeout_){
    // The injector unwraps the underscores (_) from around the parameter names when matching
    $compile = _$compile_;
    $rootScope = _$rootScope_;
    $document = _$document_;
    $templateManager = templateManager;
    $screenService = ScreenService;
    $timeout = _$timeout_;
  }));

  it('ZhClick should   call the method when the click dom event is called', function() {
    var element = $compile("<a data-zh-click='runed = true;'></a>")($rootScope);
    $rootScope.$digest();
    element.click();
    $rootScope.$digest();
    expect($rootScope.runed).toEqual(true);
  });


  it('ZhPostRepeat should exec when varable $last is true in the scope', function() {
    $rootScope.$last = true;
    var element = $compile("<a zh-post-repeat='runed = true;'></a>")($rootScope);
    $rootScope.$digest();
    expect($rootScope.runed).toEqual(true);
  });


  it('zhTouchstart should call the method when the click dom event is called', function() {
    var element = $compile("<a data-zh-touchstart='runed = true;'></a>")($rootScope);
    $rootScope.$digest();
    element.click();
    $rootScope.$digest();
    expect($rootScope.runed).toEqual(true);
  });

  it('zhTouchstart should call the method when the click dom event is called even if data-zh-stop-propagation-click is present!', function() {
    var element = $compile("<a data-zh-touchstart='runed = true;' data-zh-stop-propagation-click='true'></a>")($rootScope);
    $rootScope.$digest();
    element.click();
    $rootScope.$digest();
    expect($rootScope.runed).toEqual(true);
  });

  it('zhLongTap should call the method when the touchstart dom event is called and after 750ms the event touchend is not called!', function(done) {
    $rootScope.$parrent = {};
    var element = $compile("<a data-zh-long-tap='runed = true;'></a>")($rootScope);
    $rootScope.$digest();
    element.trigger('touchstart');
    $rootScope.$digest();
    $timeout(function(){
      $rootScope.$digest();
      expect($rootScope.runed).toEqual(true);
      done();
    }, 800);
    $timeout.flush();
    $rootScope.$digest();
  });

  it('zhLongTap should not call the method when the touchstart dom event is called and the event touchend is called in less than 750ms!', function(done) {
    $rootScope.$parrent = {};
    var element = $compile("<a data-zh-long-tap='runed = true;' on-touch-end='touchEndCalled = true' data-zh-stop-propagation-click='true'></a>")($rootScope);
    $rootScope.$digest();
    element.trigger('touchstart');
    element.trigger('touchend');
    $rootScope.$digest();
    $timeout(function(){
      $rootScope.$digest();
      expect($rootScope.runed).toEqual(undefined);
      expect($rootScope.touchEndCalled).toEqual(true);
      done();
    }, 800);
    $timeout.flush();
  });

  it('zhLongTap should not call the method when the touchstart dom event is called and the event touchmove is called in less than 750ms!', function(done) {
    $rootScope.$parrent = {};
    var element = $compile("<a data-zh-long-tap='runed = true;' on-touch-end='touchEndCalled = true' data-zh-stop-propagation-click='true'></a>")($rootScope);
    $rootScope.$digest();
    element.trigger('touchstart');
    element.trigger('touchmove');
    $rootScope.$digest();
    $timeout(function(){
      $rootScope.$digest();
      expect($rootScope.runed).toEqual(undefined);
      done();
    }, 800);
    $timeout.flush();
  });  

  it('zhLongTap should not call the method when the touchstart dom event is called and the event touchend is called in less than 750ms, even when with preventClick', function(done) {
    $rootScope.$parrent = {};
    var element = $compile("<a data-zh-long-tap='runed = true;' on-touch-end='touchEndCalled = true' data-zh-stop-propagation-click='true'></a>")($rootScope);
    $rootScope.$digest();
    element.trigger('touchstart');
    $rootScope.preventClick = true;//Should stay in the middle
    element.trigger('touchend');
    $rootScope.$digest();
    $timeout(function(){
      $rootScope.$digest();
      expect($rootScope.runed).toEqual(undefined);
      expect($rootScope.touchEndCalled).toEqual(true);
      done();
    }, 800);
    $timeout.flush();
  });  

  it('zhBack should call the method onBack in container when it is present!', function() {
    var onBackCalled = false;
    $templateManager.container = {
      onBack : function(){
        onBackCalled  = true;
      }
    };
    var element = $compile("<a data-zh-back></a>")($rootScope);
    $rootScope.$digest();
    element.click();
    $rootScope.$digest();
    expect(onBackCalled).toEqual(true);
  });

  it('zhBack should call the method goBack in ScreenService when the method onBack is not in container!', function() {
    var onBackCalled = false;
    $screenService.goBack = function(){onBackCalled = true;};
    $templateManager.container = {};
    var element = $compile("<a data-zh-back></a>")($rootScope);
    $rootScope.$digest();
    element.click();
    $rootScope.$digest();
    expect(onBackCalled).toEqual(true);
  });

  it('zhBack should call the method goBack in ScreenService when the method onBack is not in container. Event in mobile!', function() {
    var onBackCalled = false;
    Zepto.os.phone = true;
    $screenService.goBack = function(){onBackCalled = true;};
    $templateManager.container = {};
    var element = $compile("<a data-zh-back></a>")($rootScope);
    $rootScope.$digest();
    element.trigger('touchstart');
    $rootScope.$digest();
    expect(onBackCalled).toEqual(true);
  });  

  it('zhWidgetEvents should call the method onEnter in widget when the event is present!', function() {
    var onEnterCalled = false;
    $rootScope.widget = {
      onEnter : function() {
        onEnterCalled = true;
      }
    };
    var element = $compile("<a data-zh-widget-events></a>")($rootScope);
    $rootScope.$digest();
    expect(onEnterCalled).toEqual(true);
  });

  it('zhWidgetEvents should shoud works even if there is no widget in  the scope!', function() {
    var onEnterCalled = false;
    expect(function(){
      var element = $compile("<a data-zh-widget-events></a>")($rootScope);
      $rootScope.$digest();      
    }).not.toThrow();
  });

  it('zhWidgetEvents should call the method onEnter in any element when specified!', function() {
    var onEnterCalled = false;
    $rootScope.bar = {
      onEnter : function() {
        onEnterCalled = true;
      }
    };
    var element = $compile("<a data-zh-widget-events='bar'></a>")($rootScope);
    $rootScope.$digest();
    expect(onEnterCalled).toEqual(true);
  });    
  
});